@@ -147,7 +147,8 @@ static void     gdm_slave_usr2_handler (int sig);
 static void     gdm_slave_exit (gint status, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 static void     gdm_child_exit (gint status, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 static gint     gdm_slave_exec_script (GdmDisplay *d, const gchar *dir,
-				       const char *login, struct passwd *pwent);
+				       const char *login, struct passwd *pwent,
+				       gboolean pass_stdout);
 static gchar *  gdm_parse_enriched_login (const gchar *s, GdmDisplay *display);
 static void	gdm_slave_handle_notify (const char *msg);
 
@@ -319,7 +320,8 @@ setup_automatic_session (GdmDisplay *display, const char *name)
 
 	/* Run the init script. gdmslave suspends until script
 	 * has terminated */
-	gdm_slave_exec_script (display, GdmDisplayInit, NULL, NULL);
+	gdm_slave_exec_script (display, GdmDisplayInit, NULL, NULL,
+			       FALSE /* pass_stdout */);
 
 	gdm_debug ("setup_automatic_session: DisplayInit script finished");
 
@@ -393,7 +395,7 @@ gdm_slave_whack_greeter (void)
 	/* Wait for the greeter to really die, the check is just
 	 * being very anal, the pid is always set to something */
 	if (d->greetpid > 0)
-		waitpid (d->greetpid, 0, 0); 
+		ve_waitpid_no_signal (d->greetpid, 0, 0); 
 	d->greetpid = 0;
 
 	gdm_slave_send_num (GDM_SOP_GREETPID, 0);
@@ -745,6 +747,16 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 {
 	pid_t pid;
 
+	/* Set the busy cursor */
+	if (d->dsp != NULL) {
+		Cursor xcursor = XCreateFontCursor (d->dsp, GDK_WATCH);
+		XDefineCursor (d->dsp,
+			       DefaultRootWindow (d->dsp),
+			       xcursor);
+		XFreeCursor (d->dsp, xcursor);
+		XSync (d->dsp, False);
+	}
+
 	gdm_sigchld_block_push ();
 	gdm_sigterm_block_push ();
 	pid = d->sesspid = fork ();
@@ -752,10 +764,19 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 	gdm_sigchld_block_pop ();
 
 	if (pid < 0) {
+		/* return left pointer */
+		Cursor xcursor = XCreateFontCursor (d->dsp, GDK_LEFT_PTR);
+		XDefineCursor (d->dsp,
+			       DefaultRootWindow (d->dsp),
+			       xcursor);
+		XFreeCursor (d->dsp, xcursor);
+		XSync (d->dsp, False);
+
 		/* can't fork, damnit */
 		display->sesspid = 0;
 		return;
 	}
+
 	if (pid == 0) {
 		char **argv;
 		/* child */
@@ -791,6 +812,9 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 
 		openlog ("gdm", LOG_PID, LOG_DAEMON);
 
+		if (chdir ("/root") != 0)
+			chdir ("/");
+
 		/* exec the configurator */
 		argv = ve_split (GdmConfigurator);
 		if (argv != NULL &&
@@ -827,7 +851,7 @@ run_config (GdmDisplay *display, struct passwd *pwent)
 			/* must use the pid var here since sesspid might get
 			 * zeroed between the check and here by sigchld
 			 * handler */
-			waitpid (pid, 0, 0);
+			ve_waitpid_no_signal (pid, 0, 0);
 		display->sesspid = 0;
 	}
 }
@@ -843,12 +867,14 @@ restart_the_greeter (void)
 	if (greet) {
 		gdm_sigchld_block_push ();
 
+		gdm_slave_greeter_ctl_no_ret (GDM_SAVEDIE, "");
+
 		greet = FALSE;
 
-		if (d->greetpid > 0) {
-			if (kill (d->greetpid, SIGTERM) == 0)
-				waitpid (d->greetpid, 0, 0); 
-		}
+		/* Wait for the greeter to really die, the check is just
+		 * being very anal, the pid is always set to something */
+		if (d->greetpid > 0)
+			ve_waitpid_no_signal (d->greetpid, 0, 0); 
 		d->greetpid = 0;
 
 		gdm_slave_send_num (GDM_SOP_GREETPID, 0);
@@ -1053,7 +1079,7 @@ gdm_slave_wait_for_login (void)
 
 /* If path starts with a "trusted" directory, don't sanity check things */
 /* This is really somewhat "outdated" as we now really want things in
- * the picture dir or in ~/.gnome/photo */
+ * the picture dir or in ~/.gnome2/photo */
 static gboolean
 is_in_trusted_pic_dir (const char *path)
 {
@@ -1073,6 +1099,7 @@ run_pictures (void)
 	size_t bytes;
 	struct passwd *pwent;
 	char *picfile;
+	char *picdir;
 	FILE *fp;
 	char *cfgdir;
 
@@ -1100,14 +1127,14 @@ run_pictures (void)
 		setegid (pwent->pw_gid);
 		seteuid (pwent->pw_uid);
 
-		/* Sanity check on ~user/.gnome/gdm */
-		cfgdir = g_strconcat (pwent->pw_dir, "/.gnome/gdm", NULL);
+		/* Sanity check on ~user/.gnome2/gdm */
+		cfgdir = g_strconcat (pwent->pw_dir, "/.gnome2/gdm", NULL);
 		if (gdm_file_check ("run_pictures", pwent->pw_uid,
 				    cfgdir, "gdm", TRUE, GdmUserMaxFile,
 				    GdmRelaxPerms)) {
 			char *cfgstr;
 
-			cfgstr = g_strconcat ("=", pwent->pw_dir, "/.gnome/gdm=/face/picture", NULL);
+			cfgstr = g_strconcat ("=", pwent->pw_dir, "/.gnome2/gdm=/face/picture", NULL);
 			picfile = gnome_config_get_string (cfgstr);
 			g_free (cfgstr);
 
@@ -1160,12 +1187,20 @@ run_pictures (void)
 
 		/* Nothing found yet */
 		if (picfile == NULL) {
-			picfile = g_strconcat (pwent->pw_dir, "/.gnome/photo", NULL);
+			picfile = g_strconcat (pwent->pw_dir, "/.gnome2/photo", NULL);
+			picdir = g_strconcat (pwent->pw_dir, "/.gnome2", NULL);
+			if (access (picfile, F_OK) != 0) {
+				g_free (picfile);
+				picfile = g_strconcat (pwent->pw_dir, "/.gnome/photo", NULL);
+				g_free (picdir);
+				picdir = g_strconcat (pwent->pw_dir, "/.gnome", NULL);
+			}
 			if (access (picfile, F_OK) != 0) {
 				seteuid (0);
 				setegid (GdmGroupId);
 
 				g_free (picfile);
+				g_free (picdir);
 				picfile = g_strconcat (GdmGlobalFaceDir, "/",
 						       response, NULL);
 
@@ -1191,14 +1226,12 @@ run_pictures (void)
 				g_free (picfile);
 				continue;
 			}
-			g_free (picfile);
 
-			/* Sanity check on ~user/.gnome/photo */
-			picfile = g_strconcat (pwent->pw_dir, "/.gnome", NULL);
+			/* Sanity check on ~user/.gnome[2]/photo */
 			if ( ! gdm_file_check ("run_pictures", pwent->pw_uid,
-					       picfile, "photo", TRUE, GdmUserMaxFile,
+					       picdir, "photo", TRUE, GdmUserMaxFile,
 					       GdmRelaxPerms)) {
-				g_free (picfile);
+				g_free (picdir);
 
 				seteuid (0);
 				setegid (GdmGroupId);
@@ -1206,8 +1239,7 @@ run_pictures (void)
 				gdm_slave_greeter_ctl_no_ret (GDM_READPIC, "");
 				continue;
 			}
-			g_free (picfile);
-			picfile = g_strconcat (pwent->pw_dir, "/.gnome/photo", NULL);
+			g_free (picdir);
 		}
 
 		if (stat (picfile, &s) < 0) {
@@ -1277,7 +1309,8 @@ gdm_slave_greeter (void)
     gdm_debug ("gdm_slave_greeter: Running greeter on %s", d->name);
     
     /* Run the init script. gdmslave suspends until script has terminated */
-    gdm_slave_exec_script (d, GdmDisplayInit, NULL, NULL);
+    gdm_slave_exec_script (d, GdmDisplayInit, NULL, NULL,
+			   FALSE /* pass_stdout */);
 
     /* Open a pipe for greeter communications */
     if (pipe (pipe1) < 0 || pipe (pipe2) < 0) 
@@ -1635,7 +1668,8 @@ gdm_slave_chooser (void)
 		gdm_slave_exit (DISPLAY_ABORT, _("gdm_slave_chooser: Can't init pipe to gdmchooser"));
 
 	/* Run the init script. gdmslave suspends until script has terminated */
-	gdm_slave_exec_script (d, GdmDisplayInit, NULL, NULL);
+	gdm_slave_exec_script (d, GdmDisplayInit, NULL, NULL,
+			       FALSE /* pass_stdout */);
 
 	/* Fork. Parent is gdmslave, child is greeter process. */
 	gdm_sigchld_block_push ();
@@ -1737,7 +1771,7 @@ gdm_slave_chooser (void)
 		gdm_sigchld_block_push ();
 		/* wait for the chooser to die */
 		if (d->chooserpid > 0)
-			waitpid (d->chooserpid, 0, 0);
+			ve_waitpid_no_signal (d->chooserpid, 0, 0);
 		d->chooserpid  = 0;
 		gdm_sigchld_block_pop ();
 
@@ -1810,7 +1844,7 @@ gdm_get_sessions (struct passwd *pwent)
 
 	/* we know this exists already, code has already created it,
 	 * when checking the gsm saved options */
-	cfgdir = g_strconcat (pwent->pw_dir, "/.gnome", NULL);
+	cfgdir = g_strconcat (pwent->pw_dir, "/.gnome2", NULL);
 
 	/* We cannot be absolutely strict about the session
 	 * permissions, since by default they will be writable
@@ -1820,12 +1854,12 @@ gdm_get_sessions (struct passwd *pwent)
 	if (session_relax_perms == 0)
 		session_relax_perms = 1;
 
-	/* Sanity check on ~user/.gnome/session */
+	/* Sanity check on ~user/.gnome2/session */
 	session_ok = gdm_file_check ("gdm_get_sessions", pwent->pw_uid,
 				     cfgdir, "session",
 				     TRUE, GdmSessionMaxFile,
 				     session_relax_perms);
-	/* Sanity check on ~user/.gnome/session-options */
+	/* Sanity check on ~user/.gnome2/session-options */
 	options_ok = gdm_file_check ("gdm_get_sessions", pwent->pw_uid,
 				     cfgdir, "session-options",
 				     TRUE, GdmUserMaxFile,
@@ -1838,7 +1872,7 @@ gdm_get_sessions (struct passwd *pwent)
 
 		cfgstr = g_strconcat
 			("=", pwent->pw_dir,
-			 "/.gnome/session-options=/Options/CurrentSession",
+			 "/.gnome2/session-options=/Options/CurrentSession",
 			 NULL);
 		def = gnome_config_get_string (cfgstr);
 		if (def == NULL)
@@ -1856,7 +1890,7 @@ gdm_get_sessions (struct passwd *pwent)
 	got_def = FALSE;
 	if (session_ok) {
 		char *sessfile = g_strconcat (pwent->pw_dir,
-					      "/.gnome/session", NULL);
+					      "/.gnome2/session", NULL);
 		FILE *fp = fopen (sessfile, "r");
 		if (fp == NULL &&
 		    GdmGnomeDefaultSession != NULL) {
@@ -2058,6 +2092,8 @@ session_child_run (struct passwd *pwent,
 	const char *shell = NULL;
 	Display *disp;
 
+	gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+
 	disp = XOpenDisplay (d->name);
 	if (disp != NULL) {
 		XSetInputFocus (disp, PointerRoot,
@@ -2065,6 +2101,53 @@ session_child_run (struct passwd *pwent,
 		XCloseDisplay (disp);
 	}
 
+	if (strcmp (session, GDM_SESSION_FAILSAFE_GNOME) == 0 ||
+	    strcmp (session, GDM_SESSION_FAILSAFE_XTERM) == 0 ||
+	    /* hack */
+	    g_ascii_strcasecmp (session, "Failsafe") == 0) {
+		failsafe = TRUE;
+	}
+
+	/* Here we setup our 0,1,2 descriptors, we do it here
+	 * nowdays rather then later on so that we get errors even
+	 * from the PreSession script */
+        /* Log all output from session programs to a file,
+	 * unless in failsafe mode which needs to work when there is
+	 * no diskspace as well */
+	if ( ! failsafe && home_dir_ok) {
+		logfd = open (g_strconcat (home_dir, "/.xsession-errors", NULL),
+			      O_CREAT|O_TRUNC|O_WRONLY, 0644);
+		if (logfd != -1) {
+			dup2 (logfd, 1);
+			dup2 (logfd, 2);
+			close (logfd);
+		} else {
+			close (1);
+			close (2);
+			open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
+			open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+			gdm_error (_("%s: Could not open ~/.xsession-errors"),
+				   "run_session_child");
+		}
+	} else {
+		close (1);
+		close (2);
+		open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
+		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	}
+
+	close (0);
+	open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
+
+	/* Run the PreSession script */
+	if (gdm_slave_exec_script (d, GdmPreSession,
+				   login, pwent,
+				   TRUE /* pass_stdout */) != EXIT_SUCCESS &&
+	    /* ignore errors in failsafe modes */
+	    ! failsafe) 
+		/* If script fails reset X server and restart greeter */
+		gdm_child_exit (DISPLAY_REMANAGE,
+				_("gdm_slave_session_start: Execution of PreSession script returned > 0. Aborting."));
 
 	gdm_clearenv ();
 
@@ -2155,7 +2238,7 @@ session_child_run (struct passwd *pwent,
 	
 	if (usrcfgok && savesess && home_dir_ok) {
 		gchar *cfgstr = g_strconcat ("=", home_dir,
-					     "/.gnome/gdm=/session/last", NULL);
+					     "/.gnome2/gdm=/session/last", NULL);
 		gnome_config_set_string (cfgstr, save_session);
 		need_config_sync = TRUE;
 		g_free (cfgstr);
@@ -2163,7 +2246,7 @@ session_child_run (struct passwd *pwent,
 	
 	if (usrcfgok && savelang && home_dir_ok) {
 		gchar *cfgstr = g_strconcat ("=", home_dir,
-					     "/.gnome/gdm=/session/lang", NULL);
+					     "/.gnome2/gdm=/session/lang", NULL);
 		/* we chose the system default language so wipe the lang key */
 		if (def_language)
 			gnome_config_clean_key (cfgstr);
@@ -2177,7 +2260,7 @@ session_child_run (struct passwd *pwent,
 	    savegnomesess &&
 	    gnome_session != NULL &&
 	    home_dir_ok) {
-		gchar *cfgstr = g_strconcat ("=", home_dir, "/.gnome/session-options=/Options/CurrentSession", NULL);
+		gchar *cfgstr = g_strconcat ("=", home_dir, "/.gnome2/session-options=/Options/CurrentSession", NULL);
 		gnome_config_set_string (cfgstr, gnome_session);
 		need_config_sync = TRUE;
 		g_free (cfgstr);
@@ -2190,13 +2273,7 @@ session_child_run (struct passwd *pwent,
 
 	closelog ();
 
-	gdm_close_all_descriptors (0 /* from */, -1 /* except */);
-
-	/* No error checking here - if it's messed the best response
-         * is to ignore & try to continue */
-	open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
-	open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-	open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	gdm_close_all_descriptors (3 /* from */, -1 /* except */);
 
 	openlog ("gdm", LOG_PID, LOG_DAEMON);
 	
@@ -2300,21 +2377,6 @@ session_child_run (struct passwd *pwent,
 		shell = "/bin/sh";
 	}
 
-        /* Log all output from session programs to a file,
-	 * unless in failsafe mode which needs to work when there is
-	 * no diskspace as well */
-	if ( ! failsafe && home_dir_ok) {
-		logfd = open (g_strconcat (home_dir, "/.xsession-errors", NULL),
-			      O_CREAT|O_TRUNC|O_WRONLY, 0644);
-		if (logfd != -1) {
-			dup2 (logfd, 1);
-			dup2 (logfd, 2);
-		} else {
-			gdm_error (_("%s: Could not open ~/.xsession-errors"),
-				   "run_session_child");
-		}
-	}
-
 	/* just a stupid test, the below would fail, but this gives a better
 	 * message */
 	if (strcmp (shell, "/sbin/nologin") == 0 ||
@@ -2336,7 +2398,33 @@ session_child_run (struct passwd *pwent,
 			   "dialog window."));
 	} else {
 		char *exec = g_strconcat ("exec ", sesspath, NULL);
+		char *shellbase = g_path_get_basename (shell);
+		/* FIXME: this is a hack currently this is all screwed up
+		 * so I won't run the users shell unless it's one of the
+		 * "listed" ones, I'll just run bash or sh,
+		 * that's a bit evil but in the end it works out better in
+		 * fact.  In the future we will do our own login setup
+		 * stuff */
+		if (strcmp (shellbase, "sh") != 0 &&
+		    strcmp (shellbase, "bash") == 0 &&
+		    strcmp (shellbase, "tcsh") == 0 &&
+		    strcmp (shellbase, "ksh") == 0 &&
+		    strcmp (shellbase, "pdksh") == 0 &&
+		    strcmp (shellbase, "zsh") == 0 &&
+		    strcmp (shellbase, "csh") == 0 &&
+		    strcmp (shellbase, "ash") == 0 &&
+		    strcmp (shellbase, "bsh") == 0 &&
+		    strcmp (shellbase, "bash2") == 0) {
+			if (access ("/bin/bash", R_OK|X_OK) == 0)
+				shell = "/bin/bash";
+			else if (access ("/bin/bash2", R_OK|X_OK) == 0)
+				shell = "/bin/bash2";
+			else
+				shell = "/bin/sh";
+		}
 		execl (shell, "-", "-c", exec, NULL);
+		/* nutcase fallback */
+		execl ("/bin/sh", "-", "-c", exec, NULL);
 
 		gdm_error (_("gdm_slave_session_start: Could not start session `%s'"), sesspath);
 		gdm_error_box
@@ -2416,19 +2504,19 @@ gdm_slave_session_start (void)
 
     if (home_dir_ok) {
 	    char *cfgdir;
-	    /* Check if ~user/.gnome exists. Create it otherwise. */
-	    cfgdir = g_strconcat (home_dir, "/.gnome", NULL);
+	    /* Check if ~user/.gnome2 exists. Create it otherwise. */
+	    cfgdir = g_strconcat (home_dir, "/.gnome2", NULL);
 
 	    if (stat (cfgdir, &statbuf) == -1) {
 		    mkdir (cfgdir, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
 		    chmod (cfgdir, S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
 	    }
 
-	    /* Sanity check on ~user/.gnome/gdm */
+	    /* Sanity check on ~user/.gnome2/gdm */
 	    usrcfgok = gdm_file_check ("gdm_slave_session_start", pwent->pw_uid,
 				       cfgdir, "gdm", TRUE, GdmUserMaxFile,
 				       GdmRelaxPerms);
-	    /* Sanity check on ~user/.gnome/session-options */
+	    /* Sanity check on ~user/.gnome2/session-options */
 	    sessoptok = gdm_file_check ("gdm_slave_session_start", pwent->pw_uid,
 					cfgdir, "session-options", TRUE, GdmUserMaxFile,
 					/* We cannot be absolutely strict about the
@@ -2446,13 +2534,13 @@ gdm_slave_session_start (void)
     if (usrcfgok) {
 	gchar *cfgstr;
 
-	cfgstr = g_strconcat ("=", home_dir, "/.gnome/gdm=/session/last", NULL);
+	cfgstr = g_strconcat ("=", home_dir, "/.gnome2/gdm=/session/last", NULL);
 	usrsess = gnome_config_get_string (cfgstr);
 	if (usrsess == NULL)
 		usrsess = g_strdup ("");
 	g_free (cfgstr);
 
-	cfgstr = g_strconcat ("=", home_dir, "/.gnome/gdm=/session/lang", NULL);
+	cfgstr = g_strconcat ("=", home_dir, "/.gnome2/gdm=/session/lang", NULL);
 	usrlang = gnome_config_get_string (cfgstr);
 	if (usrlang == NULL)
 		usrlang = g_strdup ("");
@@ -2548,15 +2636,6 @@ gdm_slave_session_start (void)
     if (GdmKillInitClients)
 	    gdm_server_whack_clients (d);
 
-    /* setup some env for PreSession script */
-    gnome_setenv ("DISPLAY", d->name, TRUE);
-
-    /* If script fails reset X server and restart greeter */
-    if (gdm_slave_exec_script (d, GdmPreSession,
-			       login, pwent) != EXIT_SUCCESS) 
-	gdm_slave_exit (DISPLAY_REMANAGE,
-			_("gdm_slave_session_start: Execution of PreSession script returned > 0. Aborting."));
-
     /* Setup cookie -- We need this information during cleanup, thus
      * cookie handling is done before fork()ing */
 
@@ -2596,9 +2675,12 @@ gdm_slave_session_start (void)
 
     if (strcmp (session, GDM_SESSION_FAILSAFE_GNOME) == 0 ||
 	strcmp (session, GDM_SESSION_FAILSAFE_XTERM) == 0 ||
-	strcmp (session, "Failsafe") == 0 /* hack */)
+	g_ascii_strcasecmp (session, "Failsafe") == 0 /* hack */)
 	    failsafe = TRUE;
 
+    /* Write out the Xservers file */
+    gdm_slave_send_num (GDM_SOP_WRITE_X_SERVERS, 0 /* bogus */);
+
     /* don't completely rely on this, the user
      * could reset time or do other crazy things */
     session_start_time = time (NULL);
@@ -2616,7 +2698,6 @@ gdm_slave_session_start (void)
 	gdm_slave_exit (DISPLAY_ABORT, _("gdm_slave_session_start: Error forking user session"));
 	
     case 0:
-
 	/* Never returns */
 	session_child_run (pwent,
 			   home_dir,
@@ -2648,11 +2729,11 @@ gdm_slave_session_start (void)
     if (d->sesspid > 0) {
 	    gdm_slave_send_num (GDM_SOP_SESSPID, pid);
 
-	    d->sesspid = 0;
-
 	    gdm_sigchld_block_pop ();
 
-	    waitpid (pid, NULL, 0);
+	    ve_waitpid_no_signal (pid, NULL, 0);
+
+	    d->sesspid = 0;
     } else {
 	    gdm_sigchld_block_pop ();
     }
@@ -2698,6 +2779,7 @@ static void
 gdm_slave_session_stop (pid_t sesspid)
 {
     struct passwd *pwent;
+    char *x_servers_file;
     char *local_login;
 
     local_login = login;
@@ -2710,16 +2792,44 @@ gdm_slave_session_stop (pid_t sesspid)
 
     gdm_debug ("gdm_slave_session_stop: %s on %s", local_login, d->name);
 
-    if (sesspid > 0)
-	    kill (- (sesspid), SIGTERM);
+    /* Note we use the info in the structure here since if we get passed
+     * a 0 that means the process is already dead.
+     * FIXME: Maybe we should waitpid here, note make sure this will
+     * not create a hang! */
+    gdm_sigchld_block_push ();
+    if (d->sesspid > 0)
+	    kill (- (d->sesspid), SIGTERM);
+    gdm_sigchld_block_pop ();
 
     gdm_verify_cleanup (d);
     
     pwent = getpwnam (local_login);	/* PAM overwrites our pwent */
 
-    /* Execute post session script */
-    gdm_debug ("gdm_slave_session_cleanup: Running post session script");
-    gdm_slave_exec_script (d, GdmPostSession, local_login, pwent);
+    x_servers_file = g_strconcat (GdmServAuthDir,
+				  "/", d->name, ".Xservers", NULL);
+
+    /* if there was a session that ran, run the PostSession script */
+    if (sesspid != 0) {
+	    /* setup some env for PostSession script */
+	    gnome_setenv ("DISPLAY", d->name, TRUE);
+	    gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
+
+	    gnome_setenv ("X_SERVERS", x_servers_file, TRUE);
+	    if (d->type == TYPE_XDMCP)
+		    gnome_setenv ("REMOTE_HOST", d->hostname, TRUE);
+
+	    /* Execute post session script */
+	    gdm_debug ("gdm_slave_session_cleanup: Running post session script");
+	    gdm_slave_exec_script (d, GdmPostSession, local_login, pwent,
+				   FALSE /* pass_stdout */);
+
+	    gnome_unsetenv ("X_SERVERS");
+	    if (d->type == TYPE_XDMCP)
+		    gnome_unsetenv ("REMOTE_HOST");
+    }
+
+    unlink (x_servers_file);
+    g_free (x_servers_file);
 
     if (pwent == NULL) {
 	    return;
@@ -2787,7 +2897,7 @@ gdm_slave_term_handler (int sig)
 		greet = FALSE;
 		gdm_debug ("gdm_slave_term_handler: Whacking greeter");
 		if (kill (pid, sig) == 0)
-			waitpid (pid, 0, 0); 
+			ve_waitpid_no_signal (pid, 0, 0); 
 	} else if (login != NULL) {
 		gdm_slave_session_stop (d->sesspid);
 		gdm_slave_session_cleanup ();
@@ -2798,7 +2908,7 @@ gdm_slave_term_handler (int sig)
 		d->chooserpid = 0;
 		gdm_debug ("gdm_slave_term_handler: Whacking chooser");
 		if (kill (pid, sig) == 0)
-			waitpid (pid, 0, 0); 
+			ve_waitpid_no_signal (pid, 0, 0); 
 	}
 
 	gdm_debug ("gdm_slave_term_handler: Whacking server");
@@ -3005,7 +3115,7 @@ gdm_slave_xioerror_handler (Display *disp)
 		d->greetpid = 0;
 		greet = FALSE;
 		if (kill (pid, SIGINT) == 0)
-			waitpid (pid, 0, 0); 
+			ve_waitpid_no_signal (pid, 0, 0); 
 	} else if (login != NULL) {
 		gdm_slave_session_stop (d->sesspid);
 		gdm_slave_session_cleanup ();
@@ -3015,7 +3125,7 @@ gdm_slave_xioerror_handler (Display *disp)
 		pid_t pid = d->chooserpid;
 		d->chooserpid = 0;
 		if (kill (pid, SIGINT) == 0)
-			waitpid (pid, 0, 0); 
+			ve_waitpid_no_signal (pid, 0, 0); 
 	}
     
 	gdm_error (_("gdm_slave_xioerror_handler: Fatal X error - Restarting %s"), d->name);
@@ -3201,13 +3311,14 @@ gdm_child_exit (gint status, const gchar *format, ...)
 
 static gint
 gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
-		       struct passwd *pwent)
+		       struct passwd *pwent, gboolean pass_stdout)
 {
     pid_t pid;
     gchar *script, *defscript;
     const char *scr;
     gchar **argv;
     gint status;
+    char *x_servers_file;
 
     if (!d || !dir)
 	return EXIT_SUCCESS;
@@ -3215,9 +3326,9 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
     script = g_strconcat (dir, "/", d->name, NULL);
     defscript = g_strconcat (dir, "/Default", NULL);
 
-    if (! access (script, R_OK|X_OK)) {
+    if (access (script, R_OK|X_OK) == 0) {
 	    scr = script;
-    } else if (! access (defscript, R_OK|X_OK))  {
+    } else if (access (defscript, R_OK|X_OK) == 0)  {
 	    scr = defscript;
     } else {
 	    g_free (script);
@@ -3232,13 +3343,19 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
     case 0:
         closelog ();
 
-	gdm_close_all_descriptors (0 /* from */, -1 /* except */);
+	close (0);
+	open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
 
-        /* No error checking here - if it's messed the best response
-         * is to ignore & try to continue */
-        open ("/dev/null", O_RDONLY); /* open stdin - fd 0 */
-        open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
-        open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	if ( ! pass_stdout) {
+		close (1);
+		close (2);
+		/* No error checking here - if it's messed the best response
+		 * is to ignore & try to continue */
+		open ("/dev/null", O_RDWR); /* open stdout - fd 1 */
+		open ("/dev/null", O_RDWR); /* open stderr - fd 2 */
+	}
+
+	gdm_close_all_descriptors (3 /* from */, -1 /* except */);
 
 	openlog ("gdm", LOG_PID, LOG_DAEMON);
 
@@ -3262,6 +3379,14 @@ gdm_slave_exec_script (GdmDisplay *d, const gchar *dir, const char *login,
 	        gnome_setenv ("SHELL", "/bin/sh", TRUE);
         }
 
+	/* some env for use with the Pre and Post scripts */
+	x_servers_file = g_strconcat (GdmServAuthDir,
+				      "/", d->name, ".Xservers", NULL);
+	gnome_setenv ("X_SERVERS", x_servers_file, TRUE);
+	g_free (x_servers_file);
+	if (d->type == TYPE_XDMCP)
+		gnome_setenv ("REMOTE_HOST", d->hostname, TRUE);
+
 	/* Runs as root, uses server authfile */
 	gnome_setenv ("XAUTHORITY", d->authfile, TRUE);
         gnome_setenv ("DISPLAY", d->name, TRUE);
